package nori

import (
	"fmt"
	"image"
	"image/draw"

	"git.tuxpa.in/a/zlog/log"
	"github.com/phrozen/blend"
	"gitlab.com/gfxlabs/gfximg/apng"
)

func (n *Nori) ShowAnimation() *apng.APNG {
	g := n.Gawi
	a := &apng.APNG{
		Frames: make([]apng.Frame, 0, len(g.Images)),
	}
	for i := 0; i < len(g.Images); i++ {
		fr := apng.Frame{
			Image:            g.Images[i].Img,
			DelayNumerator:   uint16(g.Images[i].Delay),
			DelayDenominator: 1000,
		}
		a.Frames = append(a.Frames, fr)
	}
	return a
}

func (n *Nori) ExportAnimation() (*apng.APNG, error) {
	g := n.Gawi
	a := &apng.APNG{
		Frames: make([]apng.Frame, 0, len(g.Images)),
	}
	images := make([]*image.NRGBA64, 0, len(n.Animations))
	for i := 0; i < len(n.Animations); i++ {
		if len(n.Animations[i].Frames) < 1 {
			continue
		}
		planes := n.Animations[i].Frames[0].Planes
		var canvasRect image.Rectangle
		for _, plane := range planes {
			if int(plane.BitmapId) >= len(n.Gawi.Images) {
				log.Printf("could not find bitmap %d, only have %d", plane.BitmapId, len(n.Gawi.Images))
			}
			bitmap := n.Gawi.Images[plane.BitmapId]
			pt := image.Pt(int(plane.PlaneX), int(plane.PlaneY)) // where to put the point
			rc := bitmap.Img.Bounds()
			canvasRect = canvasRect.Union(rc.Add(pt))
		}
		img := image.NewNRGBA64(canvasRect)
		for _, plane := range planes {
			bitmap := n.Gawi.Images[plane.BitmapId]
			transparent := false
			//flipx
			if plane.RenderFlag&1 != 0 {
			}
			//flipy
			if plane.RenderFlag&2 != 0 {
			}
			// is transparent
			if plane.RenderFlag&0x20 != 0 {
				transparent = true
			}
			_ = transparent
			pt := image.Pt(int(plane.PlaneX), int(plane.PlaneY)) // where to put the point
			src := bitmap.Img
			sr := src.Bounds()
			r := image.Rectangle{pt, pt.Add(sr.Size())}
			switch plane.Blend {
			case BlendMode_Alpha:
			case BlendMode_Mul, BlendMode_Mul7:
				blend.BlendImage(img, bitmap.Img, blend.Multiply)
			case BlendMode_Add, BlendMode_Add8:
				blend.BlendImage(img, bitmap.Img, blend.Add)
			case BlendMode_InvertMul, BlendMode_InvertMul5:
				blend.BlendImage(img, bitmap.Img, blend.Multiply)
			case BlendMode_None:
			default:
				return nil, fmt.Errorf("unknown blend mode: %d", plane.Blend)
			}
			draw.Draw(img,
				r,
				src,
				sr.Min,
				draw.Over,
			)
		}
		if len(planes) > 0 {
			images = append(images, img)

		}
	}
	or := images[0].Bounds()
	for _, realFrame := range images {
		or = realFrame.Bounds().Union(or)
	}
	// TODO: this is bugged
	for i, realFrame := range images {
		big := image.NewNRGBA64(or)
		draw.Draw(
			big,
			big.Rect,
			realFrame,
			realFrame.Bounds().Min,
			draw.Src,
		)
		fr := apng.Frame{
			Image:            big,
			DisposeOp:        apng.DISPOSE_OP_PREVIOUS,
			BlendOp:          apng.BLEND_OP_OVER,
			DelayDenominator: 1000,
		}
		if g.Images[i].Delay != 0 {
			fr.DelayNumerator = uint16(g.Images[i].Delay)
		}
		a.Frames = append(a.Frames, fr)
	}
	return a, nil
}
